package com.persagy.apm.energy.report.common.service.impl;

import com.persagy.apm.common.model.dto.PoemsPuser;
import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.common.DataConstants;
import com.persagy.apm.energy.report.common.dto.QueryPlatformParamDTO;
import com.persagy.apm.energy.report.common.service.CommonService;
import com.persagy.apm.energy.report.common.service.PlatformService;
import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.SpringContextHolder;
import com.persagy.apm.energy.report.common.vo.PlatformAreaInfoVO;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.StatisticItems;
import com.persagy.apm.energy.report.monthly.config.type.constant.enums.BelongTypeEnum;
import com.persagy.apm.energy.report.monthly.detail.business.model.Platform;
import com.persagy.apm.energy.report.saasweb.model.dto.QueryProjectDTO;
import com.persagy.apm.energy.report.saasweb.model.vo.PoemsProjectVO;
import com.persagy.apm.energy.report.saasweb.service.impl.SaasWebClientWrapper;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

@Api(value = "公共service实现类")
@Service
public class CommonServiceImpl implements CommonService {

    @Autowired
    SaasWebClientWrapper saasWebClientWrapper;

    @Autowired
    SpringContextHolder holder;

    @Override
    public List<String> getPjNameByPjId(List<String> pjIdList) {
        if (CollectionUtils.isEmpty(pjIdList)) {
            return new ArrayList<>();
        }
        QueryProjectDTO queryProjectDTO = new QueryProjectDTO();
        PoemsPuser poemsPuser = new PoemsPuser();
        poemsPuser.setUserId(DataConstants.USER_ID);
        poemsPuser.setLoginDevice(DataConstants.LOGIN_DEVICE);
        poemsPuser.setPd(DataConstants.PD);
        queryProjectDTO.setPuser(poemsPuser);
        PoemsFeignResponse<PoemsProjectVO> projectsResponse = saasWebClientWrapper.queryProjects(queryProjectDTO);
        List<PoemsProjectVO> projectVOList = projectsResponse.getContent();
        if (!CollectionUtils.isEmpty(projectVOList)) {
            List<String> pjNameList = new ArrayList<>();
            for (PoemsProjectVO poemsProjectVO : projectVOList) {
                if (!pjIdList.contains(poemsProjectVO.getProjectId())) {
                    continue;
                }
                pjNameList.add(poemsProjectVO.getProjectLocalName());
            }
            return pjNameList;
        }
        return new ArrayList<>();
    }

    @Override
    public Platform getPlatform(StatisticItems statisticItems, QueryPlatformParamDTO queryPlatformParamDTO) {
        String serviceName = statisticItems.getItemCode() + statisticItems.getItemSource() + "WebService";
        PlatformService platformService = holder.getPlatformService(serviceName);
        Platform platform = new Platform();
        platform.setItemCode(statisticItems.getItemCode());
        platform.setModelName(statisticItems.getItemName());
        if (queryPlatformParamDTO.getBelongType().equals(BelongTypeEnum.PROJECT.getType())) {
            platform.setMonthlySummaries(platformService.getPjPlatformRate(queryPlatformParamDTO));
        } else if (queryPlatformParamDTO.getBelongType().equals(BelongTypeEnum.AREA.getType())) {
            // 合格标准
            String standardValueStr = statisticItems.getStandardValue();
            Double standardValue = DataUtils.parseDouble(standardValueStr);
            platform.setStandardValue(standardValue);
            platform.setQualifyFormula(statisticItems.getQualifyFormula());
            platform.setStandardFlag(statisticItems.getStandardFlag());
            PlatformAreaInfoVO platformAreaInfoVO = platformService.getAreaPlatformRate(statisticItems, queryPlatformParamDTO);
            if (null != platformAreaInfoVO) {
                platform.setSatisfiedCount(platformAreaInfoVO.getSatisfiedCount());
                platform.setUnsatisfiedCount(platformAreaInfoVO.getUnsatisfiedCount());
                platform.setUnsatisfiedProjects(platformAreaInfoVO.getUnsatisfiedProjects());
            }
        }
        return platform;
    }
}
