package com.persagy.apm.energy.report.monthly.outline.service.builder;

import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.detail.business.model.Platform;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.CostItemVO;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.CostVO;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.GroupInfo;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.PowerVO;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.saasweb.model.vo.ReportProjectVO;

import java.util.Date;
import java.util.List;

/**
 * 生成商业报告建造者接口
 *
 * @author lixing
 * @version V1.0 2021/5/30 10:26 下午
 **/
public interface IBusinessReportBuilder {
    /**
     * 生成报告
     *
     * @param reportOutline 报告outline对象
     * @author lixing
     * @version V1.0 2021/5/24 4:09 下午
     */
    void generateReport(ReportOutline reportOutline);

    /**
     * 删除之前的报告详情信息
     *
     * @param detailId 报告详情id
     * @author lixing
     * @version V1.0 2021/5/31 4:19
     */
    void deletePreviousReportDetail(String detailId);

    /**
     * 在营项目用电量信息
     *
     * @param reportOutline 报告outline对象
     * @return 在营项目用电量信息
     * @author lixing
     * @version V1.0 2021/5/31 4:21 下午
     */
    PowerVO getOpenPowerInfo(ReportOutline reportOutline);

    /**
     * 可同比项目用电量信息
     *
     * @param reportOutline 报告outline对象
     * @return 可同比项目用电量信息
     * @author lixing
     * @version V1.0 2021/5/31 4:21 下午
     */
    PowerVO getComparePowerInfo(ReportOutline reportOutline);

    /**
     * 在营项目费用信息
     *
     * @param reportOutline 报告outline对象
     * @return 在营项目费用信息
     * @author lixing
     * @version V1.0 2021/5/31 4:21 下午
     */
    CostVO getOpenCostInfo(ReportOutline reportOutline);

    /**
     * 可同比项目费用信息
     *
     * @param reportOutline 报告outline对象
     * @return 在营项目费用信息
     * @author lixing
     * @version V1.0 2021/5/31 4:21 下午
     */
    CostVO getCompareCostInfo(ReportOutline reportOutline);

    /**
     * 可同比项目费用信息
     *
     * @param reportOutline 报告outline对象
     * @return 在营项目费用信息
     * @author lixing
     * @version V1.0 2021/5/31 4:21 下午
     */
    List<Platform> getPlatforms(ReportOutline reportOutline);

    /**
     * 成功时更新报告概要
     *
     * @param reportOutline 需要更新的报告概要
     * @author lixing
     * @version V1.0 2021/5/31 4:26 下午
     */
    void updateReportOutlineInfoWhenSuccess(ReportOutline reportOutline);

    /**
     * 失败时更新报告概要
     *
     * @param reportOutline 需要更新的报告概要
     * @author lixing
     * @version V1.0 2021/5/31 4:26 下午
     */
    void updateReportOutlineInfoWhenFailure(ReportOutline reportOutline);
}
