package com.persagy.apm.energy.report.monthly.outline.service.impl;

import cn.hutool.core.thread.ExecutorBuilder;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.energy.report.monthly.config.type.constant.enums.BelongTypeEnum;
import com.persagy.apm.energy.report.monthly.config.type.constant.enums.BuildingTypeEnum;
import com.persagy.apm.energy.report.monthly.config.type.model.ReportType;
import com.persagy.apm.energy.report.monthly.config.type.service.IReportTypeService;
import com.persagy.apm.energy.report.monthly.outline.constants.enums.ReportStateEnum;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * 消费消息线程池
 *
 * @author lixing
 * @version V1.0 2021/5/12 2:53 下午
 **/
@Component
@Slf4j
public class GenerateReportThreadPool {
    @Autowired
    private BusinessProjectReportGenerator businessProjectReportGenerator;
    @Autowired
    private BusinessAreaReportGenerator businessAreaReportGenerator;
    @Autowired
    private IReportTypeService reportTypeService;

    private final ExecutorService es;

    public GenerateReportThreadPool() {
        // 初始化线程池
        es = ExecutorBuilder.create()
                .setCorePoolSize(5)
                .setMaxPoolSize(10)
                .setWorkQueue(new LinkedBlockingQueue<>(100))
                .setHandler(new ThreadPoolExecutor.CallerRunsPolicy())
                .build();
    }

    /**
     * 生成报告
     *
     * @param reportOutline 报告outline对象
     * @author lixing
     * @version V1.0 2021/5/24 4:09 下午
     */
    public void generateReport(ReportOutline reportOutline) {
        String userId = PoemsContext.getContext().getUserId();
        String pd = PoemsContext.getContext().getPd();
        String loginDevice = PoemsContext.getContext().getLoginDevice();
        es.execute(() -> {
            try {
                PoemsContext.setContext(userId, loginDevice, pd);
                String reportTypeId = reportOutline.getReportTypeId();
                ReportType reportType = reportTypeService.queryReportTypeDetail(reportTypeId);
                if (BuildingTypeEnum.BUSINESS.getType().equals(reportType.getBuildingType())) {
                    if (BelongTypeEnum.PROJECT.getType().equals(reportType.getBelongType())) {
                        synchronized (reportOutline.getId().intern()) {
                            businessProjectReportGenerator.generateReport(reportOutline);
                        }
                    } else {
                        synchronized (reportOutline.getId().intern()) {
                            businessAreaReportGenerator.generateReport(reportOutline);
                        }
                    }
                }
            } catch (Exception e) {
                log.error("生成报告失败", e);
                reportOutline.setStatus(ReportStateEnum.BUILD_FAILED.getType());
            }
        });
    }
}
